CREATE TABLE change_nodes(
    change_id INTEGER NOT NULL,
    node_id INTEGER NOT NULL,
    hub_change INTEGER NOT NULL DEFAULT 0,
    project_change INTEGER NOT NULL DEFAULT 0,
    PRIMARY KEY(change_id,node_id) ON CONFLICT IGNORE,
    FOREIGN KEY(change_id) REFERENCES changes(id) ON DELETE CASCADE,
    FOREIGN KEY(node_id) REFERENCES nodes(id) ON DELETE CASCADE
) WITHOUT ROWID;
